/********************************************************************************
 * IRQ PA non fa altro che settare una nuova funzione di interrupt su PA. Ci
 * che si vuole fare  anche evitare il sovrabbondare di interrupt dovuto al
 * fatto che il pulsante  connesso in logica inversa al processore.
 * Version:  0.1
 * Author:   Calzo
 * Arch:     AXIS ETRAX 100 LX Risc 32 bit processor
 * License:  DUAL BSD/GPL
 * ------------------------------------------------------------------------------
 * Questo modulo non ha bisogno di un file di interfaccia in /dev
 * ------------------------------------------------------------------------------
 * NOTE: 
 * ******************************************************************************/

#include <linux/interrupt.h>
#include <linux/module.h>
#include <asm/io.h>      //provide access to GPIO port and other peripherals on ETRAX100LX

MODULE_LICENSE("DUAL GPL/BSD");
MODULE_VERSION("0.1");

#define IRQ_PA_MASK 0x01 //interrupt sul pulsante

static DEFINE_SPINLOCK(gpio_lock_irq);

/***** FUNZIONE DI INTERRUPT ******
 * In questa funzione viene disabilitata temporaneamente la possibilit
 * di ricevere altri interrupt fino a cha non  soddisfatta la funzione
 * seguente
 * ********************************/
static irqreturn_t
  irqPA_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
   *R_IRQ_MASK1_CLR = 0;
   *R_PORT_PA_DATA = 0;
   printk(KERN_ALERT "calzo - irq servito\n");
   return IRQ_HANDLED; //IRQ_NONE;
}	

/****************************************
 * INITIALIZATION & CLEANUP functions
 ****************************************/
/** Funzione di inizializzazione del modulo
 * */
int irqPA_init_module(void)
{
   if (request_irq(PA_IRQ_NBR, irqPA_interrupt,
		   SA_INTERRUPT,"calzo gpio PA interrupt", NULL)) 
     {
	printk(KERN_CRIT "err: PA irq for gpio (calzo)\n");
	return -ERESTART;
     }
   *R_IRQ_MASK1_SET = IRQ_PA_MASK;
   
   return 0;
}

/** Funzione di scaricamento del modulo
 * */
void irqPA_cleanup_module(void)
{
   spin_lock_irq(&gpio_lock_irq);
   *R_IRQ_MASK1_SET = 0;
   *R_IRQ_MASK1_CLR = 0;
   free_irq(PA_IRQ_NBR, NULL);
   spin_unlock_irq(&gpio_lock_irq);
}
      

/***************************************
 * INIT & EXIT
 ***************************************/
module_init(irqPA_init_module);
module_exit(irqPA_cleanup_module);
